@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2002-2015 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2016 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2002-2015 Horde LLC
+ * @copyright 2002-2016 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Crypt
  */
@@ -20,7 +20,7 @@
  * @author    Michael Slusarz <slusarz@horde.org>
  * @author    Jan Schneider <jan@horde.org>
  * @category  Horde
- * @copyright 2002-2015 Horde LLC
+ * @copyright 2002-2016 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Crypt
  */
@@ -88,7 +88,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
                 if ($ret !== false) {
                     return $ret;
                 }
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         throw new Horde_Crypt_Exception(
@@ -99,6 +99,12 @@ class Horde_Crypt_Pgp extends Horde_Crypt
     /**
      * Returns information on a PGP data block.
      *
+     * If the data block contains multiple keys, only the first is returned. To
+     * return all keys of this block, use pgpPacketInformationMultiple()
+     * instead.
+     *
+     * @see pgpPacketInformationMultiple()
+     *
      * @param string $pgpdata  The PGP data block.
      *
      * @return array  An array with information on the PGP data block. If an
@@ -149,7 +155,32 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->packetInfo($pgpdata);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
+        }
+
+        return array();
+    }
+
+    /**
+     * Returns all information on a PGP data block.
+     *
+     * @since Horde_Crypt 2.7.0
+     * @see pgpPacketInformation()
+     *
+     * @param string $pgpdata  The PGP data block.
+     *
+     * @return array  An array with information on the PGP data block. The
+     *                array contains one or more entries as returned from
+     *                pgpPacketInformation().
+     */
+    public function pgpPacketInformationMultiple($pgpdata)
+    {
+        $this->_initDrivers();
+
+        foreach ($this->_backends as $val) {
+            try {
+                return $val->packetInfoMultiple($pgpdata);
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         return array();
@@ -374,7 +405,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->getSignersKeyId($text);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         return null;
@@ -482,7 +513,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->getFingerprintsFromKey($pgpdata);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         return array();
@@ -503,7 +534,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->getPublicKeyFromPrivateKey($data);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         return null;
@@ -557,7 +588,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->$func($text, $params);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         throw new Horde_Crypt_Exception($error);
@@ -621,7 +652,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->$func($text, $params);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         throw new Horde_Crypt_Exception(
@@ -647,7 +678,7 @@ class Horde_Crypt_Pgp extends Horde_Crypt
         foreach ($this->_backends as $val) {
             try {
                 return $val->isEncryptedSymmetrically($text);
-            } catch (Exception $e) {}
+            } catch (Horde_Crypt_Exception $e) {}
         }
 
         throw new Horde_Crypt_Exception(
